/*
   File: dcg_alloc.h
   Provides basic allocation routines and garbage collection
   while lessening the overhead caused by malloc and free.
   Note that an initial call to init_alloc () is required.

   CVS ID: "$Id: dcg_alloc.h,v 1.6 2005/02/24 14:59:35 marcs Exp $"
*/
#ifndef IncDcgAlloc
#define IncDcgAlloc

/* Local includes */
#include <dcg.h>

/* Basic allocation routines */
char *dcg_malloc (int size);
char *dcg_attach (char *ptr);
void dcg_detach (char **ptr);
char *dcg_predetach (char **ptr);
void dcg_cknonshared (char *ptr);
void dcg_mkconstant (char *ptr);

/* Derived routines */
char *dcg_calloc (int nr, int size);
void dcg_realloc (char **ptr, int size);
void dcg_recalloc (char **ptr, int nr, int size);

/* Convenience macro */
#define nonrec_detach(x) dcg_detach((char **) x)

/* Initialization and inspection of the garbage collector */
void init_alloc ();
void report_alloc ();

#endif /* IncDcgAlloc */
